@@ -1,16 +1,16 @@
 vardchanges <- function(Y1, H1, PSU1, w_final1, id1,
-                     Dom1 = NULL,
-                     Z1 = NULL,
+                     Dom1 = NULL, Z1 = NULL,
                      country1, period1,
                      dataset1 = NULL,
                      Y2, H2, PSU2, w_final2, id2,
-                     Dom2 = NULL,
-                     Z2 = NULL,
+                     Dom2 = NULL, Z2 = NULL,
                      country2, period2,
                      dataset2 = NULL,
                      confidence=0.95) {
  
   ### Checking
+  linratio = FALSE
+
   if(!is.numeric(confidence) || length(confidence) != 1 || confidence[1] < 0 || confidence[1] > 1) {
           stop("'confidence' must be a numeric value in [0,1]")  }
 
@@ -76,6 +76,7 @@ vardchanges <- function(Y1, H1, PSU1, w_final1, id1,
   Y1 <- data.table(Y1, check.names=TRUE)
   n1 <- nrow(Y1)
   m1 <- ncol(Y1)
+  if (!all(sapply(Y1, is.numeric))) stop("'Y1' must be numerical")
   if (any(is.na(Y1))) stop("'Y1' has unknown values")
   if (is.null(names(Y1))) stop("'Y1' must be colnames")
   
@@ -131,17 +132,15 @@ vardchanges <- function(Y1, H1, PSU1, w_final1, id1,
     if ((is.null(Dom1))!=(is.null(Dom2))) stop("'Dom1' and 'Dom2' must be same column count") 
   }
   
-  # Z1
   if (!is.null(Z1)) {
-    Z1 <- data.table(Z1)
+    Z1 <- data.table(Z1, check.names=TRUE)
     if (nrow(Z1) != n1) stop("'Z1' and 'Y1' must be equal row count")
     if (ncol(Z1) != m1) stop("'Z1' and 'Y1' must be equal column count")
     if (any(is.na(Z1))) stop("'Z1' has unknown values")
     if (is.null(names(Z1))) stop("'Z1' must be colnames")
     if ((is.null(Z1))!=(is.null(Z2))) stop("'Z1' and 'Z2' must be same column count") 
   }
-      
-  
+ 
   if(!is.null(dataset2)) {
       dataset2 <- data.frame(dataset2)
       aY <- Y2
@@ -176,10 +175,11 @@ vardchanges <- function(Y1, H1, PSU1, w_final1, id1,
                                 names(w_final2) <- aw_final }}
       if(!is.null(Z2)) {
           aZ <- Z2
-          if (min(Z2 %in% names(dataset2))!=1) stop("'Z2' does not exist in 'dataset2'!")
-          if (min(Z2 %in% names(dataset2))==1) {
-                                Z2 <-data.frame(dataset2[, aZ], check.names=FALSE, stringsAsFactors=FALSE)
+          if (min(Z2 %in% names(dataset1))!=1) stop("'Z2' does not exist in 'dataset1'!")
+          if (min(Z2 %in% names(dataset1))==1) {
+                                Z2 <-data.frame(dataset1[, aZ], check.names=FALSE, stringsAsFactors=FALSE)
                                 names(Z2) <- aZ }}
+
       if(!is.null(country2)) {
           acountry <- country2
           if (min(country2 %in% names(dataset2))!=1) stop("'country2' does not exist in 'dataset2'!")
@@ -204,11 +204,12 @@ vardchanges <- function(Y1, H1, PSU1, w_final1, id1,
   Y2 <- data.table(Y2, check.names=TRUE)
   n2 <- nrow(Y2)
   m2 <- ncol(Y2)
+  if (!all(sapply(Y2, is.numeric))) stop("'Y2' must be numerical")   
   if (any(is.na(Y2))) stop("'Y2' has unknown values")
   if (is.null(names(Y2))) stop("'Y2' must be colnames")
   if (m1!=m2) stop("'Y1' and 'Y2' must be equal column count")
   if (!all(names(Y1) %in% names(Y2))) stop("'Y1' and 'Y2' must be same colnames")
- 
+
   # H2
   H2 <- data.table(H2)
   if (nrow(H2) != n2) stop("'H2' length must be equal with 'Y2' row count")
@@ -273,7 +274,7 @@ vardchanges <- function(Y1, H1, PSU1, w_final1, id1,
 
   # Z2
   if (!is.null(Z2)) {
-    Z2 <- data.table(Z2)
+    Z2 <- data.table(Z2, check.names=TRUE)
     if (nrow(Z2) != n2) stop("'Z2' and 'Y2' must be equal row count")
     if (ncol(Z2) != m2) stop("'Z2' and 'Y2' must be equal column count")
     if (any(is.na(Z2))) stop("'Z2' has unknown values")
@@ -282,26 +283,33 @@ vardchanges <- function(Y1, H1, PSU1, w_final1, id1,
     if (ncol(Z1)!=ncol(Z2)) stop("'Z1' and 'Z2' must be same column count")
     if (!all(names(Z1) %in% names(Z2))) stop("'Z1' and 'Z2' must be same colnames")
   }
- 
+  
   data1 <- vardcros(Y=Y1, H=H1, PSU=PSU1, w_final=w_final1,
                     id=id1, Dom=Dom1, Z=Z1, country=country1,
                     period=period1, dataset=NULL,
-                    meanY=TRUE, withperiod=TRUE,
-                    netchanges=TRUE, confidence = confidence)
-
-  H1 <- PSU1 <- w_final1 <- id1 <- NULL
-  Y1 <- names(Y1)
-  dataset1 <- Dom1 <- Z1 <- country1 <- NULL
+                    linratio=linratio, household_level_max=TRUE,
+                    withperiod=TRUE, netchanges=TRUE, 
+                    confidence = confidence)
 
+  namesY <- namesZ <- H1 <- PSU1 <- w_final1 <-  NULL
+  id1 <- dataset1 <- Dom1 <- Z1 <- country1 <- NULL
+  
   data2 <- vardcros(Y=Y2, H=H2, PSU=PSU2, w_final=w_final2,
                     id=id2, Dom=Dom2, Z=Z2, country=country2,
                     period=period2, dataset=NULL,
-                    meanY=TRUE, withperiod=TRUE,
-                    netchanges=TRUE, confidence = confidence)
-  H2 <- names(H2)
-  Y2 <- names(Y2)
-  PSU2 <- w_final2 <- id2 <- NULL
-  dataset2 <- Dom2 <- Z2 <- country2 <- NULL
+                    linratio=linratio, household_level_max=TRUE,
+                    withperiod=TRUE, netchanges=TRUE, 
+                    confidence = confidence)
+
+  Dom <- names(Dom2)
+  H <- names(H2)
+  Y <- names(Y2)
+  nz <- ifelse(!is.null(Z2), 1, 0)
+  np <- ifelse(!is.null(Dom2), ncol(Dom2), 0)
+  np <- 2 + np + nz
+
+  namesYs <- namesZs <- PSU2 <- w_final2 <- NULL
+  id2 <- dataset2 <- Dom2 <- Z2 <- country2 <- NULL
   N <- grad1 <- grad2 <- country <- rot01 <- rot02 <- stratasf <- NULL
   name1 <- num1num1 <- num1 <- den1den1 <- den1 <- num2num2 <- NULL
   num2 <- den2den2 <- den2 <-  num1den1 <- num1num2 <- num1den2 <- NULL
@@ -315,16 +323,17 @@ vardchanges <- function(Y1, H1, PSU1, w_final1, id1,
 
   var_grad1 <- data1$var_grad
   var_grad2 <- data2$var_grad
-  var_grad1[, (c(names(period1), "name2")):=NULL]
-  var_grad2[, (c(names(period2), "name2")):=NULL]
-  var_grad1[, grad1:=-grad1]
-  var_grad1[, grad2:=-grad2]
-  setnames(var_grad1, names(var_grad1)[-c(1:2)], paste0(names(var_grad1)[-c(1:2)], "_1"))
-  setnames(var_grad2, names(var_grad2)[-c(1:2)], paste0(names(var_grad2)[-c(1:2)], "_2"))
+  var_grad1[, (names(period1)):=NULL]
+  var_grad2[, (names(period2)):=NULL]
+  var_grad1[!is.null(grad1), grad1:=-grad1]
+  var_grad1[!is.null(grad1), grad2:=-grad2]
+
+  setnames(var_grad1, names(var_grad1)[-c(1:np)], paste0(names(var_grad1)[-c(1:np)], "_1"))
+  setnames(var_grad2, names(var_grad2)[-c(1:np)], paste0(names(var_grad2)[-c(1:np)], "_2"))
   var_grad1[, country:=as.character(country)]
   var_grad2[, country:=as.character(country)]
-  setkeyv(var_grad1, names(var_grad1)[1:2])
-  setkeyv(var_grad2, names(var_grad2)[1:2])
+  setkeyv(var_grad1, names(var_grad1)[1:np])
+  setkeyv(var_grad2, names(var_grad2)[1:np])
   var_grad <- merge(var_grad1, var_grad2, all=T)
   var_grad1 <- var_grad2 <- NULL
 
@@ -332,10 +341,10 @@ vardchanges <- function(Y1, H1, PSU1, w_final1, id1,
   data2 <- data2$data_net_changes
   data1[, (names(period1)):=NULL]
   data2[, (names(period2)):=NULL]
-  
-  nrowv <- nrow(var_grad[, .N, by="name1"])
-  period <- names(data1)[3+1:nrowv]
-  echnames <- names(data1)[(3+nrowv+1):ncol(data1)]
+  main <- ifelse(!is.null(var_grad$namesZ), "namesY", c("namesY", Dom))
+  nrowv <- nrow(var_grad)
+  if ((!linratio)&(4+nrowv<ncol(data1))) namesZ <- names(data1)[(4+nrowv):ncol(data1)] 
+  period <- names(data1)[4:ncol(data1)]
   setnames(data1, names(data1)[-c(1:3)], paste0(names(data1)[-c(1:3)], "_1"))
   setnames(data2, names(data2)[-c(1:3)], paste0(names(data2)[-c(1:3)], "_2"))
   period2 <- names(data2)[4:ncol(data2)]
@@ -352,7 +361,7 @@ vardchanges <- function(Y1, H1, PSU1, w_final1, id1,
    }
   recode.NA(data, c(paste0(period,"_1"), paste0(period,"_2")))
 
-  dataH <- data[[H2]]
+  dataH <- data[[H]]
   dataH <- factor(dataH)
   if (length(levels(dataH))==1) { data[, stratasf:= 1]
                                 dataH <- "stratasf"
@@ -360,19 +369,15 @@ vardchanges <- function(Y1, H1, PSU1, w_final1, id1,
                                 data <- cbind(data, dataH)
                                 dataH <- names(dataH) }
 
-  echnames <- rep(echnames, nrowv/length(echnames))
-
-  meanpop=TRUE
+  den1 <- den2 <- NULL
   fit <- lapply(1:nrowv, function(i) {
-                   i=1
            fits <- lapply(split(data, data$country), function(d) {
-                   i=1
-                   d=data
                    y1 <- paste0(period[i], "_1")
                    y2 <- paste0(period[i], "_2")
-                   if (meanpop) { z1 <- paste0(",", toString(echnames[i]), "_1") 
-	                            z2 <- paste0(",", toString(echnames[i]), "_2")
-                                } else z1 <- z2 <- ""
+                   if (!is.null(var_grad$namesZ)) { 
+                                    z1 <- paste0(",", toString(period[i + nrowv]), "_1") 
+	                              z2 <- paste0(",", toString(period[i + nrowv]), "_2")
+                               } else z1 <- z2 <- ""
                    vect <- c("rot01*", "rot02*", "rot01*rot02*")
 
                    funkc <- as.formula(paste0("cbind(", trim(toString(y1)), z1, ", ", 
@@ -383,44 +388,63 @@ vardchanges <- function(Y1, H1, PSU1, w_final1, id1,
                                                                         "rot01*rot02*", toString(x))))),
                                                                         collapse= "+"))) 
                    res <- data.table(lm(funkc, data=d)$res)
-                   setnames(res, names(res), c("num1", "den1", "num2", "den2"))
-                   res[, name1:=period[i]]
+                   if (!is.null(var_grad$namesZ)) { 
+                         setnames(res, names(res), c("num1", "den1", "num2", "den2"))
+                       } else setnames(res, names(res), c("num1", "num2"))
+                   res[, namesY:=period[i]]
+                   res[!is.null(var_grad$namesZ), namesZ:=period[i + nrowv]]
                    res[, num1num1:=num1 * num1]
-                   res[, den1den1:=den1 * den1]
+                   res[!is.null(den1), den1den1:=den1 * den1]
                    res[, num2num2:=num2 * num2]
-                   res[, den2den2:=den2 * den2]
-                   res[, num1den1:=num1 * den1]
+                   res[!is.null(den2), den2den2:=den2 * den2]
+                   res[!is.null(den1), num1den1:=num1 * den1]
                    res[, num1num2:=num1 * num2]
-                   res[, num1den2:=num1 * den2]
-                   res[, den1num2:=den1 * num2]
-                   res[, den1den2:=den1 * den2]
-                   res[, num2den2:=num2 * den2] 
+                   res[!is.null(den2), num1den2:=num1 * den2]
+                   res[!is.null(den1), den1num2:=den1 * num2]
+                   res[!is.null(den2), den1den2:=den1 * den2]
+                   res[!is.null(den2), num2den2:=num2 * den2] 
 
                    res <- data.table(res, d)
                    varsp <- c("num1num1", "den1den1", "num2num2", "den2den2", "num1den1",
                               "num1num2", "num1den2", "den1num2", "den1den2", "num2den2")
                    varsp <- varsp[varsp %in% names(res)]
-                   fits <- res[, lapply(.SD, sum), keyby=c("country", "name1"), .SDcols=varsp]
+                   keynames <- c("country", "namesY", "namesZ")
+                   keynames <- keynames[keynames %in% names(res)]
+                   fits <- res[, lapply(.SD, sum), keyby=keynames, .SDcols=varsp]
                 })
             data.table(do.call("rbind", fits))      
         })
    res <- data.table(do.call("rbind", fit))
-   res[, country:=as.character(country)]
-   setkeyv(res, c("country", "name1"))
-   setkeyv(var_grad, c("country", "name1"))
 
+   res[, country:=as.character(country)]
+   namesYZ <- c("namesY", "namesZ")
+   namesYZ <- namesYZ[namesYZ %in% names(res)]
+   setnames(res, namesYZ,  paste0(namesYZ, "s"))
+
+   var_grad[, namesYs:=Reduce(function(x, y)
+                              paste(x, y, sep = "__"), .SD),
+                              .SDcols=c("namesY", Dom)]
+   var_grad[!is.null(namesZ), namesZs:=Reduce(function(x, y)
+                                       paste(x, y, sep = "__"), .SD),
+                                      .SDcols=c("namesZ", Dom)]
+
+   setkeyv(res, c("country", paste0(namesYZ, "s")))
+   setkeyv(var_grad, c("country", paste0(namesYZ, "s")))
    data <- merge(res, var_grad, all=T)
    res <- fit <- var_gr <- NULL
+   data[, namesYs:=NULL]
+   data[!is.null(namesZ), namesZs:=NULL]
 
-   data[, C12:=sqrt(num1_1)*sqrt(den1_1)*num1den1/(sqrt(num1num1)*sqrt(den1den1))]
+
+   data[!is.null(namesZ), C12:=sqrt(num1_1)*sqrt(den1_1)*num1den1/(sqrt(num1num1)*sqrt(den1den1))]
    data[, C13:=sqrt(num1_1)*sqrt(num1_2)*num1num2/(sqrt(num1num1)*sqrt(num2num2))]
-   data[, C14:=sqrt(num1_1)*sqrt(den1_2)*num1den2/(sqrt(num1num1)*sqrt(den2den2))]
-   data[, C23:=sqrt(den1_1)*sqrt(num1_2)*den1num2/(sqrt(den1den1)*sqrt(num2num2))]
-   data[, C24:=sqrt(den1_1)*sqrt(den1_2)*den1num2/(sqrt(den1den1)*sqrt(den2den2))]
-   data[, C34:=sqrt(num1_2)*sqrt(den1_2)*num2den2/(sqrt(num2num2)*sqrt(den2den2))]
+   data[!is.null(namesZ), C14:=sqrt(num1_1)*sqrt(den1_2)*num1den2/(sqrt(num1num1)*sqrt(den2den2))]
+   data[!is.null(namesZ), C23:=sqrt(den1_1)*sqrt(num1_2)*den1num2/(sqrt(den1den1)*sqrt(num2num2))]
+   data[!is.null(namesZ), C24:=sqrt(den1_1)*sqrt(den1_2)*den1num2/(sqrt(den1den1)*sqrt(den2den2))]
+   data[!is.null(namesZ), C34:=sqrt(num1_2)*sqrt(den1_2)*num2den2/(sqrt(num2num2)*sqrt(den2den2))]
 
    data[, estim:=estim_1 - estim_2]
-   data[, var:= (grad1_1 * grad1_1 * num1_1) +  
+   data[!is.null(namesZ), var:= (grad1_1 * grad1_1 * num1_1) +  
                 (grad1_2 * grad1_2 * den1_1) + 
                 (grad2_1 * grad2_1 * num1_2) + 
                 (grad2_2 * grad2_2 * den1_2) + 
@@ -430,6 +454,9 @@ vardchanges <- function(Y1, H1, PSU1, w_final1, id1,
                 (grad1_2 * grad2_1 * C23) +
                 (grad1_2 * grad2_2 * C24) +
                 (grad2_1 * grad2_2 * C34))]
+
+   data[is.null(namesZ), var:= num1_1 + num1_2]
+
    data[, se:=sqrt(var)]
    data[, rse:=se/estim]
    data[, cv:=rse*100]
@@ -438,7 +465,7 @@ vardchanges <- function(Y1, H1, PSU1, w_final1, id1,
    data[, relative_margin_of_error:= tsad * cv]
    data[, CI_lower:=estim - tsad*se]
    data[, CI_upper:=estim + tsad*se]
-   data[, c("country", "name1","estim",
+   data[, c("country", Dom, namesYZ, "estim",
             "var", "se", "rse", "cv",
             "absolute_margin_of_error",
             "relative_margin_of_error",
