@@ -110,6 +110,7 @@ vardom <- function(Y, H, PSU, w_final,
   n <- nrow(Y)
   m <- ncol(Y)
   if (any(is.na(Y))) stop("'Y' has unknown values")
+  if (!all(sapply(Y, is.numeric))) stop("'Y' must be numeric values")
   if (is.null(names(Y))) stop("'Y' must be colnames")
   
   # H
@@ -133,13 +134,6 @@ vardom <- function(Y, H, PSU, w_final,
   if (!is.numeric(w_final)) stop("'w_final' must be numerical")
   if (any(is.na(w_final))) stop("'w_final' has unknown values") 
 
-  # id
-  if (is.null(id)) id <- PSU
-  id <- data.table(id)
-  if (any(is.na(id))) stop("'id' has unknown values")
-  if (nrow(id) != n) stop("'id' length must be equal with 'Y' row count")
-  if (ncol(id) != 1) stop("'id' must be 1 column data.frame, matrix, data.table")
-  if (is.null(names(id))||(names(id)=="id")) setnames(id,names(id),"ID")
 
   # period     
   if (!is.null(period)) {
@@ -151,6 +145,19 @@ vardom <- function(Y, H, PSU, w_final,
       if(any(is.na(period))) stop("'period' has unknown values")
   } 
   np <- sum(ncol(period))
+ 
+  # id
+  if (is.null(id)) id <- PSU
+  id <- data.table(id)
+  if (any(is.na(id))) stop("'id' has unknown values")
+  if (ncol(id) != 1) stop("'id' must be 1 column data.frame, matrix, data.table")
+  if (nrow(id) != n) stop("'id' length must be equal with 'Y' row count")
+  if (is.null(names(id))||(names(id)=="id")) setnames(id,names(id),"ID")
+  if (is.null(period)){ if (any(duplicated(id))) stop("'id' are duplicate values") 
+                      } else {
+                         id1 <- data.table(period, id)
+                         if (any(duplicated(id1))) stop("'id' by period are duplicate values")
+                        }
 
   # N_h
   if (!is.null(N_h)) {
@@ -159,13 +166,17 @@ vardom <- function(Y, H, PSU, w_final,
       if (!is.numeric(N_h[[ncol(N_h)]])) stop("The last column of 'N_h' should be numerical")
       if (any(is.na(N_h))) stop("'N_h' has unknown values") 
       if (is.null(names(N_h))) stop("'N_h' must be colnames")
+      if (is.null(names(N_h))) stop("'N_h' must be colnames")
+
       if (is.null(period)) {
              if (names(H) != names(N_h)[1]) stop("Strata titles for 'H' and 'N_h' is not equal")
              if (any(is.na(merge(unique(H), N_h, by=names(H), all.x = T)))) stop("'N_h' is not defined for all stratas")
+             if (any(duplicated(N_h[, head(names(N_h),-1), with=F]))) stop("Strata values for 'N_h' must be unique")
        } else { pH <- data.frame(period, H)
                 if (any(names(pH) != names(N_h)[c(1:(1+np))])) stop("Strata titles for 'period' with 'H' and 'N_h' is not equal")
                 if (any(is.na(merge(unique(pH), N_h, by=names(pH), all.x = T)))) stop("'N_h' is not defined for all stratas and periods")
-                pH <- NULL 
+                if (any(duplicated(N_h[, head(names(N_h),-1), with=F]))) stop("Strata values for 'N_h' must be unique in all periods")
+                pH <- NULL
      }
     setkeyv(N_h, names(N_h)[c(1:(1+np))])
   }
@@ -186,6 +197,7 @@ vardom <- function(Y, H, PSU, w_final,
     Z <- data.table(Z, check.names = T)
     if (nrow(Z) != n) stop("'Z' and 'Y' must be equal row count")
     if (ncol(Z) != m) stop("'Z' and 'Y' must be equal column count")
+    if (!all(sapply(Z, is.numeric))) stop("'Z' must be numeric values")
     if (any(is.na(Z))) stop("'Z' has unknown values")
     if (is.null(names(Z))) stop("'Z' must be colnames")
   }
@@ -193,6 +205,7 @@ vardom <- function(Y, H, PSU, w_final,
   # X
   if (!is.null(X)) {
     X <- data.table(X, check.names=T)
+    if (!all(sapply(X, is.numeric))) stop("'X' must be numeric values")
     if (nrow(X) != n) stop("'X' and 'Y' have different row count")
   }
 
@@ -252,7 +265,7 @@ vardom <- function(Y, H, PSU, w_final,
   if (!is.null(X)) w_design <- w_final / g else w_design <- w_final
       
   # Ratio of two totals
-  lin_outp <- variableZ <- estim <- deff_sam <- NULL
+  linratio_outp <- variableZ <- estim <- deff_sam <- NULL
   deff_est <- deff <- var_est2 <- se <- rse <- cv <- NULL
   absolute_margin_of_error <- relative_margin_of_error <- NULL
   Z1 <- CI_lower <- CI_upper <- variable <- NULL
@@ -276,7 +289,7 @@ vardom <- function(Y, H, PSU, w_final,
             Y2a <- rbindlist(lin2)[sorts]
         }
     if (any(is.na(Y2))) print("Results are calculated, but there are cases where Z = 0")
-    if (outp_lin) ratio_outp <- data.table(idper, PSU, Y2) 
+    if (outp_lin) linratio_outp <- data.table(idper, PSU, Y2) 
   } else {
           Y2 <- Y1
           Y2a <- Y1
@@ -396,6 +409,7 @@ vardom <- function(Y, H, PSU, w_final,
   all_result[, deff_sam:=var_cur_HT / var_srs_HT]
   
   # Effect of estimator
+
   all_result[, deff_est:= var_est / var_cur_HT]
   
   # Overall effect of sample design and estimator
@@ -404,8 +418,8 @@ vardom <- function(Y, H, PSU, w_final,
   all_result[, var_est2:=var_est]
   all_result[xor(is.na(var_est2), var_est2 < 0), var_est2:=NA]
   all_result[, se:=sqrt(var_est2)]
-  all_result[estim!=0, rse:= se/estim]
-  all_result[estim==0, rse:= NA]
+  all_result[(estim!=0) & !is.nan(estim), rse:= se/estim]
+  all_result[estim==0 | is.nan(estim), rse:=NA]
   all_result[, cv:= rse*100]
 
   tsad <- qnorm(0.5*(1+confidence))
@@ -452,7 +466,7 @@ vardom <- function(Y, H, PSU, w_final,
   setkeyv(all_result, c("nr_names", names(Dom), names(period)))
   all_result <- all_result[, c("variable", names(Dom), names(period), variab), with=F]
 
-  list(lin_out = lin_outp,
+  list(lin_out = linratio_outp,
        res_out = res_outp,
        all_result = all_result)
 }
